#include "raylib.h"
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <conio.h>

#define SIZE_OF_HEX 20

struct celluleDemineur{
    int line, column;
    bool hasMine;
    int nearbyMine;
    Vector2 pos;
    bool isPressed;
    bool isFlaged;
    Color couleurCase;
};

typedef struct celluleDemineur* CelluleDemineur;

bool containsMine(int line, int col, CelluleDemineur** matriceMines){
    return (matriceMines[line][col]->hasMine == true);
}

void afficherGrilleConsole(int nbLigneMatrice, int nbColonneMatrice, CelluleDemineur** matriceMines){
    for(int i=0; i<nbLigneMatrice; i++){
        for(int j=0; j<nbColonneMatrice; j++){
            printf("M%d:N%d - ", matriceMines[i][j]->hasMine, matriceMines[i][j]->nearbyMine);
        }
        printf("\n");
    }
}

CelluleDemineur* recupererVoisins(int line, int column, CelluleDemineur** matriceMines, int nbLigneMatrice, int nbColonneMatrice){
    CelluleDemineur* tabVoisins = calloc(6, sizeof(struct celluleDemineur));
    for(int i=0; i<6; i++){
        tabVoisins[i] = NULL;
    }
    
    bool linePrev = (line-1 >= 0), colPrev = (column-1 >= 0);
    bool lineNext = (line+1 < nbLigneMatrice), colNext = (column+1 < nbColonneMatrice);
    
    if(colPrev){tabVoisins[0] = matriceMines[line][column-1];}
    if(colNext){tabVoisins[1] = matriceMines[line][column+1];}
    
    if(linePrev){tabVoisins[2] = matriceMines[line-1][column];}
    if(lineNext){tabVoisins[3] = matriceMines[line+1][column];}
    
    if(line%2 == 0){
        if(linePrev && colNext){tabVoisins[4] = matriceMines[line-1][column+1];}
        if(lineNext && colNext){tabVoisins[5] = matriceMines[line+1][column+1];}
    }else{
        if(linePrev && colPrev){tabVoisins[4] = matriceMines[line-1][column-1];}
        if(lineNext && colPrev){tabVoisins[5] = matriceMines[line+1][column-1];}
    }
    
    return tabVoisins;
}

void incrementationVoisinMine(int line, int column, CelluleDemineur** matriceMines, int nbLigneMatrice, int nbColonneMatrice){
    CelluleDemineur* voisins = recupererVoisins(line, column, matriceMines, nbLigneMatrice, nbColonneMatrice);
    
    for(int i=0; i<6; i++){
        if(voisins[i] != NULL){
            voisins[i]->nearbyMine++;
        }
    }
    free(voisins);
}

void revelerVoisinsAZero(int line, int column, CelluleDemineur** matriceMines, int nbLigneMatrice, int nbColonneMatrice){
    CelluleDemineur* voisins = recupererVoisins(line, column, matriceMines, nbLigneMatrice, nbColonneMatrice);
    
    for(int i=0; i<6; i++){
        if(voisins[i] != NULL){
            if(voisins[i]->nearbyMine == 0 && !voisins[i]->isPressed){
                voisins[i]->isPressed = true;
                voisins[i]->couleurCase = WHITE;
                revelerVoisinsAZero(voisins[i]->line, voisins[i]->column, matriceMines, nbLigneMatrice, nbColonneMatrice);
            }
        }
    }
    free(voisins);
}

void generationMines(int nbMines, int nbLigneMatrice, int nbColonneMatrice, CelluleDemineur** matriceMines){
    srand(time(NULL));
    for(int i=0; i<nbMines; i++){
        int lineRand = rand()%nbLigneMatrice;
        int colRand = rand()%nbColonneMatrice;
        
        if(!containsMine(lineRand, colRand, matriceMines)){
            matriceMines[lineRand][colRand]->hasMine = true;
            matriceMines[lineRand][colRand]->nearbyMine = -9;
            incrementationVoisinMine(lineRand, colRand, matriceMines, nbLigneMatrice, nbColonneMatrice);
        }
    }
}



int main(void)
{
    // Initialization
    //--------------------------------------------------------------------------------------
    const int screenWidth = 800;
    const int screenHeight = 800;
    
    bool gameIsLost = false;
    
    //CREER MATRICE DES MINES
    int nbLigneMatrice = 10, nbColonneMatrice = 10, nbMines = 3;
    CelluleDemineur** matriceMines = malloc(nbLigneMatrice * sizeof(CelluleDemineur *));
    for(int i=0; i<nbColonneMatrice; i++)
        matriceMines[i] = malloc(nbColonneMatrice * sizeof(CelluleDemineur));

    for(int i=0; i<nbLigneMatrice; i++){
        for(int j=0; j<nbColonneMatrice; j++){
            matriceMines[i][j] = malloc(sizeof(struct celluleDemineur));
            matriceMines[i][j]->hasMine = false;
            matriceMines[i][j]->isPressed = false;
            matriceMines[i][j]->isFlaged = false;
            matriceMines[i][j]->nearbyMine = 0;
            matriceMines[i][j]->line = i;
            matriceMines[i][j]->column = j;
            matriceMines[i][j]->couleurCase = LIGHTGRAY; 
            if(i%2 == 0){
                matriceMines[i][j]->pos = (Vector2){(1.5*j+1.75)*SIZE_OF_HEX, (1.5*i+1)*SIZE_OF_HEX};
            }else{
                matriceMines[i][j]->pos = (Vector2){(1.5*j+1)*SIZE_OF_HEX, (1.5*i+1)*SIZE_OF_HEX};
            }
        }
    }
    
    //GENERATION DES MINES
    generationMines(nbMines, nbLigneMatrice, nbColonneMatrice, matriceMines);
    afficherGrilleConsole(nbLigneMatrice, nbColonneMatrice, matriceMines);
    int remainingMines = nbMines;

    InitWindow(screenWidth, screenHeight, "HexaSweeper");

    SetTargetFPS(60);               // Set our game to run at 60 frames-per-second
    //--------------------------------------------------------------------------------------

    // Main game loop
    while (!WindowShouldClose())    // Detect window close button or ESC key
    {
        // Update
        //----------------------------------------------------------------------------------
        if(IsMouseButtonPressed(MOUSE_LEFT_BUTTON)){
            matriceMines[0][0]->isPressed = true;
            if(matriceMines[0][0]->nearbyMine == 0){
                matriceMines[0][0]->couleurCase = WHITE;
                revelerVoisinsAZero(0, 0, matriceMines, nbLigneMatrice, nbColonneMatrice);
            }else if(matriceMines[0][0]->nearbyMine < 0){
                matriceMines[0][0]->couleurCase = MAROON;
                gameIsLost = true;
            }else{
                matriceMines[0][0]->couleurCase = BEIGE;
            }
        }
        
        if(IsMouseButtonPressed(MOUSE_MIDDLE_BUTTON)){
            if(!matriceMines[0][0]->isPressed){
                matriceMines[0][0]->isFlaged = !matriceMines[0][0]->isFlaged;
                if(matriceMines[0][0]->isFlaged){
                    matriceMines[0][0]->couleurCase = GREEN;
                    remainingMines--;
                }
                else{
                    matriceMines[0][0]->couleurCase = LIGHTGRAY;
                    remainingMines++;
                }
            }
            
        }
        
        // Draw
        //----------------------------------------------------------------------------------
        BeginDrawing();

            ClearBackground(RAYWHITE);
            
            
            
            for(int i=0; i<nbLigneMatrice; i++){
                for(int j=0; j<nbColonneMatrice; j++){
                    DrawPoly(matriceMines[i][j]->pos, 6, SIZE_OF_HEX, 0, GRAY);
                    DrawPoly(matriceMines[i][j]->pos, 6, SIZE_OF_HEX-5, 0, matriceMines[i][j]->couleurCase);
                    
                    if(matriceMines[i][j]->isPressed && matriceMines[i][j]->nearbyMine>0) 
                        DrawText(TextFormat("%i", matriceMines[i][j]->nearbyMine), matriceMines[i][j]->pos.x, matriceMines[i][j]->pos.y, SIZE_OF_HEX/2, BROWN);
                }
            }
            
            

        EndDrawing();
        //----------------------------------------------------------------------------------
    }

    // De-Initialization
    //--------------------------------------------------------------------------------------
    CloseWindow();        // Close window and OpenGL context
    //--------------------------------------------------------------------------------------

    return 0;
}